var _EVENT_INFO;

import analytics from './segmentWrapper';
/**
 * TCF Api Version to use
 * @type {number}
 */

var TCF_API_VERSION = 2;
/**
 * List of purpose ids needed to be able to track with all info
 * @type {Array<number>}
 */

var NEEDED_PURPOSES = [8, 10];
/**
 * Event that determines that the user has performed an action
 * @type {string}
 */

var TCF_EVENT_USER_ACTION_COMPLETE = 'useractioncomplete';
/**
 * Event that determines that the user is seeing the UI to give consent
 * @type {string}
 */

var TCF_EVENT_SHOW_UI = 'cmpuishown';
/**
 * State of user according to GDPR regarding tracking
 */

var USER_GDPR = {
  ACCEPTED: 'accepted',
  DECLINED: 'declined',
  UNKNOWN: 'unknown'
};
/**
 * Events info according
 * @type {{ [event: string]: [eventId: string, gdprValue: string]}}
 */

var EVENT_INFO = (_EVENT_INFO = {}, _EVENT_INFO[USER_GDPR.ACCEPTED] = ['Privacy Accepted', USER_GDPR.ACCEPTED], _EVENT_INFO[USER_GDPR.DECLINED] = ['Privacy Declined', USER_GDPR.DECLINED], _EVENT_INFO[USER_GDPR.UNKNOWN] = ['Privacy Impression', USER_GDPR.UNKNOWN], _EVENT_INFO);
/**
 * Define the user GDPR consents state. This value will be updated with new values
 * when the consents of the users changes.
 */

var gdprState = {
  _onChange: function _onChange(value) {},
  value: USER_GDPR.UNKNOWN,
  onChange: function onChange(callback) {
    return gdprState._onChange = callback;
  },
  get: function get() {
    return gdprState.value;
  },
  set: function set(value) {
    gdprState.value = value;

    gdprState._onChange(value);
  }
};
/**
 * Check if we're on client and tcfApi is available on window object
 * @returns {Boolean}
 */

var checkTcfIsAvailable = function checkTcfIsAvailable() {
  // first check if we're on server, as this doesn't work on server-side
  var isClient = typeof window !== 'undefined';
  if (!isClient) return false; // if we're on client, check if we have the tcfapi available

  var isTcfApiAvailable = !!window.__tcfapi;
  !isTcfApiAvailable && console.warn("[tcfTracking] window.__tcfapi is not available on client and TCF won't be tracked.");
  return isTcfApiAvailable;
};
/**
 * Check from a list of consents if user has accepted being tracked
 * @param {{[purposeId: string]: boolean}} userConsents
 * @returns {Boolean}
 */


var checkHasUserConsentedTracking = function checkHasUserConsentedTracking(userConsents) {
  return NEEDED_PURPOSES.every(function (purposeId) {
    return userConsents["" + purposeId];
  });
};
/**
 * Track a specific TCF event
 * @param {object} params
 * @param {string} params.eventId Event ID to be sent with the TCF Tracking
 * @param {string} params.gdprPrivacy Send a string telling if the gdpr has been accepted or reject
 * @return {Promise}
 */


var trackTcf = function trackTcf(_ref) {
  var eventId = _ref.eventId,
      gdprPrivacy = _ref.gdprPrivacy;
  return analytics.track('Event Fired', {
    channel: 'GDPR',
    event_id: eventId
  }, {
    gdpr_privacy: gdprPrivacy
  });
};
/**
 * Check if DMP is consented by the user using boros API
 * @return {Promise<boolean>}
 */


export var checkIsDMPReady = function checkIsDMPReady() {
  return new Promise(function (resolve) {
    if (typeof window === 'undefined') return resolve();
    if (window.__borosTcf === undefined) return resolve();

    window.__borosTcf.push(function (api) {
      api('isDmpAccepted', function (_ref2) {
        var success = _ref2.success,
            value = _ref2.value;
        var isDMPReady = success && value;
        return resolve(isDMPReady);
      });
    });
  });
};
/**
 * Get if we have user consents
 * @return {Promise<string>}
 */

export var getGdprPrivacyValue = function getGdprPrivacyValue() {
  // try to get the actual gdprPrivacyValue and just return it
  var gdprPrivacyValue = gdprState.get();
  return Promise.resolve(gdprPrivacyValue); // TODO: Disabled for now. It will be added on the next iteration
  // if (gdprPrivacyValue !== undefined) return Promise.resolve(gdprPrivacyValue)
  // // if we don't have a gdprPrivacyValue, then subscribe to it until we have a value
  // return new Promise(resolve => {
  //   gdprPrivacyValueState.onChange = gdprPrivacyValue =>
  //     resolve(gdprPrivacyValue)
  // })
};
/**
 * Check if gdprPrivacyValue is accepted
 * @return {boolean}
 */

export var checkGdprIsAccepted = function checkGdprIsAccepted(gdprPrivacyValue) {
  return gdprPrivacyValue === USER_GDPR.ACCEPTED;
};
/**
 * Init TCF Tracking User Consents with Segment
 */

export default function initTcfTracking() {
  checkTcfIsAvailable() && window.__tcfapi('addEventListener', TCF_API_VERSION, function (_ref3, success) {
    var eventStatus = _ref3.eventStatus,
        purpose = _ref3.purpose;
    if (!success) return Promise.resolve();

    if (eventStatus === TCF_EVENT_SHOW_UI) {
      var _EVENT_INFO$USER_GDPR = EVENT_INFO[USER_GDPR.UNKNOWN],
          eventId = _EVENT_INFO$USER_GDPR[0],
          gdprPrivacy = _EVENT_INFO$USER_GDPR[1];
      return trackTcf({
        eventId: eventId,
        gdprPrivacy: gdprPrivacy
      });
    }

    if (eventStatus === TCF_EVENT_USER_ACTION_COMPLETE) {
      var consents = purpose.consents;
      var hasConsents = checkHasUserConsentedTracking(consents); // get the state key according to the gdprPrivacyValue

      var gdprStateKey = hasConsents ? USER_GDPR.ACCEPTED : USER_GDPR.DECLINED;
      gdprState.set(gdprStateKey); // extract the eventId and gdprPrivacy string to send with the track

      var _EVENT_INFO$gdprState = EVENT_INFO[gdprStateKey],
          _eventId = _EVENT_INFO$gdprState[0],
          _gdprPrivacy = _EVENT_INFO$gdprState[1];
      return trackTcf({
        eventId: _eventId,
        gdprPrivacy: _gdprPrivacy
      });
    }
  });
}